from Products.generator.i18n import translate

#
# Command Translater
#
def translate(command, target="", value=""):
    """ Translate an api call into selenese command. """
    result = "|%s|%s|%s|" % (command, target, value)
    if command == 'testComplete':
        result == "|testComplete|"
    return result

from Dispatcher import Dispatcher

class SeleniumInterpreter(Dispatcher):
    """ Performs the following:
        1) Translates python API calls into Selenese commands,
        2) Sends the command string to the command dispatcher.

    Interpreter:
         _Computer Science_: A program that translates an instruction into a 
         machine language and executes it before proceeding to the next instruction.
         (source: http://dictionary.reference.com/search?q=Interpreter)
    """

    def __init__(self, id):
        Dispatcher.__init__(self)
        self.id = id
       
    def driver(self, REQUEST=None):
        """ expose dispatcher's webDriver method to web requests """
        return self.webDriver(REQUEST)
    

    def dispatchCommand(self, command, target="", value=""):
        command_string = translate(command, target, value)        
        return self.apiDriver(command_string)

    #
    # Selenium Actions       
    #
    def open(self, path):
        """ Goto the specified URL """
        return self.dispatchCommand("open", path)

    def click(self, target):
        """ Click on the target """
        return self.dispatchCommand("click", target)
        
    def clickAndWait(self, target):
        """ Click on the target and wait for a page load event """
        return self.dispatchCommand("clickAndWait", target)

    def type(self, field, value):
        """ Overwrite the text in the located text element. """
        return self.dispatchCommand("type", field, value)
        
    def typeAndWait(self, field, value):
        """ Overwrite the text in the located text element, 
        and wait for a page load event."""
        return self.dispatchCommand("typeAndWait", field, value)

    def select(self, field, value):
        """ Select the option from the located select element."""
        return self.dispatchCommand("select",field, value)       
        
    def selectAndWait(self, field, value):
        """ Select the target and wait for a page load event """
        return self.dispatchCommand("selectAndWait",field, value)

    def selectWindow(self, window):
        """ Select the named window to be the active window. """
        return self.dispatchCommand("selectWindow", window)

    def pause(self, duration): # is this needed for driven ?
        """ pause for a period of time in seconds """
        return self.dispatchCommand("pause", duration)

    def waitForValue(self, target, value):
        """ Waits for a specified input (e.g. a hidden field) to have a 
            specified value. Will succeed immediately if the input already has
            the value. Is implemented by polling for the value. 
            Warning: can block indefinitely if the input never has the specified
            value.
            target: The input field.
            value: The value the input field must have before continuing.
        """
        return self.dispatchCommand("waitForValue", target, value)

    def goBack(self):
        """ Simulates the user clicking the "back" button on their browser. """
        return self.dispatchCommand("goBack")

    def close(self):
        """Simulates the user clicking the "close" button in the titlebar of a 
           popup window."""
        return self.dispatchCommand("close")
        
    def store(self, target, value):
        """Stores the value of a parameter into a variable
           target: The value to store. This can be constructed using either 
                   variable substitution or javascript evaluation.
           value: Name of the variable to store the value into.
        """            
        return self.dispatchCommand("store", target, value)

    def storeValue(self, field, value):
       """Stores the value of an input field into a variable.
          target: The id of the input field.
          value: Name of the variable to store the field value into.
       """
       return self.dispatchCommand("storeValue", field, value)

    def storeText(self, element, value):
        """Stores the text of an element into a variable.
           target: The id of the element.
           value: Name of the variable to store the element text into.
        """
        return self.dispatchCommand("storeText", element, value)

    #
    # Selenium Checks       
    #

    def verifyLocation(self, location):
        """ Verify the location of the current page. """
        return self.dispatchCommand("verifyLocation", location)

    def assertLocation(self, location):
        """ Assert the location of the current page. 
            location: The relative location of the page
        """
        return self.dispatchCommand("assertLocation", location)

    def verifyTitle(self, title):
        """ Verify the title of the current page. """
        return self.dispatchCommand("verifyTitle", title)

    def assertTitle(self, title):
        """ Assert the title of the current page. """
        return self.dispatchCommand("assertTitle", title)

    def verifyValue(self, field, value):
        """ Verify the value of a form element. """
        return self.dispatchCommand("verifyValue", field, value)

    def assertValue(self, field, value):
        """ Assert the value of a form element. """
        return self.dispatchCommand("assertValue", field, value)

    def verifySelected(self, field, value):
        """ Verify the label of the option that is selected. """
        return self.dispatchCommand("verifySelected", field, value)

    def assertSelected(self, field, value):
        """ Assert the label of the option that is selected. """
        return self.dispatchCommand("assertSelected", field, value)

    def verifySelectOptions(self, field, values):
        """ Verify the label of all of the options in the drop=down. """
        return self.dispatchCommand("verifySelectOptions", field, values)

    def assertSelectOptions(self, field, values):
        """ Assert the label of all of the options in the drop=down. """
        return self.dispatchCommand("assertSelectOptions", field, values)

    def verifyText(self, type, text):
        """ Verifies that the entire text of the page matches the expected content."""
        return self.dispatchCommand("verifyText", type, text)

    def assertText(self, type, text):
        """ Asserts that the entire text of the page matches the expected content."""
        return self.dispatchCommand("assertText", type, text)

    def verifyAttribute(self, element, value):
        """  Verify the value of an element attribute. 
        The syntax for returning an element attribute 
        is <element-locator>@attribute-name. """
        return self.dispatchCommand("verifyAttribute", element, value)

    def assertAttribute(self, element, value):
        """  Assert the value of an element attribute. 
        The syntax for returning an element attribute 
        is <element-locator>@attribute-name. """
        return self.dispatchCommand("assertAttribute", element, value)

    def verifyTextPresent(self, text):
        """ Verifies that the specified text is present in the page content."""
        return self.dispatchCommand("verifyTextPresent", text)

    def assertTextPresent(self, text):
        """ Asserts that the specified text is present in the page content."""
        return self.dispatchCommand("assertTextPresent", text)

    def verifyTextNotPresent(self, text):
        """ Verifies that the specified text is not present in the page content."""
        return self.dispatchCommand("verifyTextNotPresent", text)

    def assertTextNotPresent(self, text):
        """ Asserts that the specified text is not present in the page content."""
        return self.dispatchCommand("assertTextNotPresent", text)

    def verifyElementPresent(self, element):
        """ Verifies that the specified element can be found. """
        return self.dispatchCommand("verifyElementPresent", element)

    def assertElementPresent(self, element):
        """ Asserts that the specified element can be found. """
        return self.dispatchCommand("assertElementPresent", element)

    def verifyElementNotPresent(self, element):
        """ Asserts that the specified element cannot be found."""
        return self.dispatchCommand("verifyElementNotPresent", element)
        
    def assertElementNotPresent(self, element):
        """ Asserts that the specified element cannot be found."""
        return self.dispatchCommand("verifyElementNotPresent", element)

    def verifyTable(self, table, value):
        """ Verifies that the text for a single cell within and HTML 
        table matches the expected content.
        The table locator syntax is table.row.column. """
        return self.dispatchCommand("verifyTable", table, value)

    def assertTable(self, table, value):
        """ Asserts that the text for a single cell within and HTML 
        table matches the expected content.
        The table locator syntax is table.row.column. """
        return self.dispatchCommand("assertTable", table, value)

    def verifyVisible(self, element):
        """ Verifies that the specified element is both present and visible. """
        return self.dispatchCommand("verifyVisible", element)

    def assertVisible(self, element):
        """ Asserts that the specified element is both present and visible. """
        return self.dispatchCommand("assertVisible", element)

    def verifyNotVisible(self, element):
        """ Verifies that the specified element is not present or not visible. """
        return self.dispatchCommand("verifyNotVisible", element)

    def assertNotVisible(self, element):
        """ Asserts that the specified element is not present or not visible. """
        return self.dispatchCommand("assertNotVisible", element)
        
    def verifyEditable(self, element):
        """ Verifies that the specified element is editable. """
        return self.dispatchCommand("verifyEditable", element)

    def assertEditable(self, element):
        """ Asserts that the specified element is editable. """
        return self.dispatchCommand("assertEditable", element)

    def verifyNotEditable(self, element):
        """ Verifies that the specified element is not editable. """
        return self.dispatchCommand("verifyNotEditable", element)

    def assertNotEditable(self, element):
        """ Asserts that the specified element is not editable. """
        return self.dispatchCommand("assertNotEditable", element)

    def assertFailureOnNext(self, element):
        """ XXX """
        return self.dispatchCommand("assertFailureOnNext", element)

    def assertErrorOnNext(self, element):
        """ XXX """
        return self.dispatchCommand("assertErrorOnNext", element)
        
    def verifyAlert(self, alert):
        """  Verifies that the supplied message was received as an alert. """
        return self.dispatchCommand("verifyAlert", alert)
        
    def assertAlert(self, alert):
        """  Asserts that the supplied message was received as an alert. """
        return self.dispatchCommand("assertAlert", alert)
        
    def verifyConfirmation(self, confirmation):
        """ Verifies that the supplied message was received as a confirmation. """
        return self.dispatchCommand("verifyConfirmation", confirmation)
        
    def assertConfirmation(self, confirmation):
        """ Asserts that the supplied message was received as a confirmation. """
        return self.dispatchCommand("assertConfirmation", confirmation)
        
    def chooseCancelOnNextConfirmation(self):
        """ Instruct Selenium to click Cancel on the next confirm dialog it encounters. """
        return self.dispatchCommand("chooseCancelOnNextConfirmation")
        
    #
    # Selenium Misc
    #
        
    def setTextField(self, field, value):
        """ Overwrite the text in the located text element. """
        return self.dispatchCommand("setTextField", field, value)        
        
    def testComplete(self):
        """ Tell the browser the test is complete """
        return self.dispatchCommand("testComplete")

    def assertFailureOnNext(self, element):
        "Assert a failure on the next command"
        return self.dispatchCommand("assertFailureOnNext")

    def assertErrorOnNext(self, element):
        "Assert an error on the next command"
        return self.dispatchCommand("assertErrorOnNext")

class PloneSeleniumInterpreter(SeleniumInterpreter):
        
    def verifyTextPresent(self, text, msgid=None, domain='plone'):
        if msgid:
            from Products.PlacelessTranslationService import translate
            text = translate(domain, 
                                          msgid, 
                                          target_language=self.language,
                                          default=text)
        return SeleniumInterpreter.verifyTextPresent(self, text) 
